@@ -65,8 +65,8 @@ namespace detail
      int k;
      const T m_x;
      T m_term;
-     expm1_series(const expm1_series&);
-     expm1_series& operator=(const expm1_series&);
+     expm1_series(const expm1_series&) = delete;
+     expm1_series& operator=(const expm1_series&) = delete;
   };
 
 template <class T, class Policy, class tag>
@@ -120,7 +120,7 @@ T expm1_imp(T x, const std::integral_constant<int, 0>&, const Policy& pol)
       if(a >= tools::log_max_value<T>())
       {
          if(x > 0)
-            return policies::raise_overflow_error<T>("boost::math::expm1<%1%>(%1%)", 0, pol);
+            return policies::raise_overflow_error<T>("boost::math::expm1<%1%>(%1%)", nullptr, pol);
          return -1;
       }
       return exp(x) - T(1);
@@ -147,7 +147,7 @@ T expm1_imp(T x, const std::integral_constant<int, 53>&, const P& pol)
       if(a >= tools::log_max_value<T>())
       {
          if(x > 0)
-            return policies::raise_overflow_error<T>("boost::math::expm1<%1%>(%1%)", 0, pol);
+            return policies::raise_overflow_error<T>("boost::math::expm1<%1%>(%1%)", nullptr, pol);
          return -1;
       }
       return exp(x) - T(1);
@@ -174,7 +174,7 @@ T expm1_imp(T x, const std::integral_constant<int, 64>&, const P& pol)
       if(a >= tools::log_max_value<T>())
       {
          if(x > 0)
-            return policies::raise_overflow_error<T>("boost::math::expm1<%1%>(%1%)", 0, pol);
+            return policies::raise_overflow_error<T>("boost::math::expm1<%1%>(%1%)", nullptr, pol);
          return -1;
       }
       return exp(x) - T(1);
@@ -183,16 +183,16 @@ T expm1_imp(T x, const std::integral_constant<int, 64>&, const P& pol)
       return x;
 
    static const float Y = 0.10281276702880859375e1f;
-   static const T n[] = { 
-      BOOST_MATH_BIG_CONSTANT(T, 64, -0.281276702880859375e-1), 
-       BOOST_MATH_BIG_CONSTANT(T, 64, 0.512980290285154286358e0), 
+   static const T n[] = {
+      BOOST_MATH_BIG_CONSTANT(T, 64, -0.281276702880859375e-1),
+       BOOST_MATH_BIG_CONSTANT(T, 64, 0.512980290285154286358e0),
        BOOST_MATH_BIG_CONSTANT(T, 64, -0.667758794592881019644e-1),
        BOOST_MATH_BIG_CONSTANT(T, 64, 0.131432469658444745835e-1),
        BOOST_MATH_BIG_CONSTANT(T, 64, -0.72303795326880286965e-3),
        BOOST_MATH_BIG_CONSTANT(T, 64, 0.447441185192951335042e-4),
        BOOST_MATH_BIG_CONSTANT(T, 64, -0.714539134024984593011e-6)
    };
-   static const T d[] = { 
+   static const T d[] = {
       BOOST_MATH_BIG_CONSTANT(T, 64, 1.0),
       BOOST_MATH_BIG_CONSTANT(T, 64, -0.461477618025562520389e0),
       BOOST_MATH_BIG_CONSTANT(T, 64, 0.961237488025708540713e-1),
@@ -217,7 +217,7 @@ T expm1_imp(T x, const std::integral_constant<int, 113>&, const P& pol)
       if(a >= tools::log_max_value<T>())
       {
          if(x > 0)
-            return policies::raise_overflow_error<T>("boost::math::expm1<%1%>(%1%)", 0, pol);
+            return policies::raise_overflow_error<T>("boost::math::expm1<%1%>(%1%)", nullptr, pol);
          return -1;
       }
       return exp(x) - T(1);
@@ -226,7 +226,7 @@ T expm1_imp(T x, const std::integral_constant<int, 113>&, const P& pol)
       return x;
 
    static const float Y = 0.10281276702880859375e1f;
-   static const T n[] = { 
+   static const T n[] = {
       BOOST_MATH_BIG_CONSTANT(T, 113, -0.28127670288085937499999999999999999854e-1),
       BOOST_MATH_BIG_CONSTANT(T, 113, 0.51278156911210477556524452177540792214e0),
       BOOST_MATH_BIG_CONSTANT(T, 113, -0.63263178520747096729500254678819588223e-1),
@@ -238,7 +238,7 @@ T expm1_imp(T x, const std::integral_constant<int, 113>&, const P& pol)
       BOOST_MATH_BIG_CONSTANT(T, 113, -0.15995603306536496772374181066765665596e-8),
       BOOST_MATH_BIG_CONSTANT(T, 113, 0.45261820069007790520447958280473183582e-10)
    };
-   static const T d[] = { 
+   static const T d[] = {
       BOOST_MATH_BIG_CONSTANT(T, 113, 1.0),
       BOOST_MATH_BIG_CONSTANT(T, 113, -0.45441264709074310514348137469214538853e0),
       BOOST_MATH_BIG_CONSTANT(T, 113, 0.96827131936192217313133611655555298106e-1),
@@ -265,9 +265,9 @@ inline typename tools::promote_args<T>::type expm1(T x, const Policy& /* pol */)
    typedef typename policies::evaluation<result_type, Policy>::type value_type;
    typedef typename policies::precision<result_type, Policy>::type precision_type;
    typedef typename policies::normalise<
-      Policy, 
-      policies::promote_float<false>, 
-      policies::promote_double<false>, 
+      Policy,
+      policies::promote_float<false>,
+      policies::promote_double<false>,
       policies::discrete_quantile<>,
       policies::assert_undefined<> >::type forwarding_policy;
 
@@ -279,7 +279,7 @@ inline typename tools::promote_args<T>::type expm1(T x, const Policy& /* pol */)
    > tag_type;
 
    detail::expm1_initializer<value_type, forwarding_policy, tag_type>::force_instantiate();
-   
+
    return policies::checked_narrowing_cast<result_type, forwarding_policy>(detail::expm1_imp(
       static_cast<value_type>(x),
       tag_type(), forwarding_policy()), "boost::math::expm1<%1%>(%1%)");
